#!/usr/bin/env python3

import os
import argparse
import yaml
import base64
import json

from kubernetes import client, config
from google.cloud import secretmanager
from google.api_core.exceptions import AlreadyExists

def sanitize_key(key):
    """
    Sanitize the key to be used as a secret name.
    Replace any special characters other than underscore and hiphen with underscore.
    """
    return "".join([c if c.isalnum() or c in ['_', '-'] else '_' for c in key])

def load_k8s_secret(namespace, secret_name):
    """
    Load a secret from a Kubernetes cluster.
    """
    config.load_kube_config()
    v1 = client.CoreV1Api()

    try:
        secret = v1.read_namespaced_secret(secret_name, namespace)
        secret_data = {k: base64.b64decode(v).decode('utf-8') for k, v in secret.data.items()}
        return secret_data
    except client.exceptions.ApiException as e:
        print(f"Exception when reading secret: {e}")
        return None

def load_k8s_configmap(namespace, configmap_name):
    """
    Load a configmap from a Kubernetes cluster.
    """
    config.load_kube_config()
    v1 = client.CoreV1Api()

    try:
        configmap = v1.read_namespaced_config_map(configmap_name, namespace)
        configmap_data = {k: v for k, v in configmap.data.items()}
        return configmap_data
    except client.exceptions.ApiException as e:
        print(f"Exception when reading secret: {e}")
        return None

def create_secret(project_id, secret_id):
    """
    Create a new secret in Secret Manager.
    """
    client = secretmanager.SecretManagerServiceClient()
    parent = f"projects/{project_id}"

    try:
        response = client.create_secret(
            request={
                "parent": parent,
                "secret_id": secret_id,
                "secret": {"replication": {"automatic": {}}},
            }
        )
        print(f"Created secret: {response.name}")
    except AlreadyExists:
        print(f"Secret {secret_id} already exists.")

def add_secret_version(project_id, secret_id, payload):
    """
    Add a new version to the existing secret with the provided payload.
    """
    client = secretmanager.SecretManagerServiceClient()
    parent = f"projects/{project_id}/secrets/{secret_id}"

    # Get the latest version of the secret
    response = client.list_secret_versions(request={"parent": parent})
    if len(list(response)) > 0:
        latest_version = list(response)[0].name
        # Check if the value of the latest version is the same as the new value
        response = client.access_secret_version(
            request={"name": latest_version}
        )
        if payload == response.payload.data.decode("UTF-8"):
            print(f"Secret version {latest_version} already exists.")
            return

    response = client.add_secret_version(
        request={
            "parent": parent,
            "payload": {
                "data": payload.encode("UTF-8"),
            },
        }
    )
    print(f"Added secret version: {response.name}")

def copy_secret_to_sm(project_id, secret_id, secret_data, multi_key=False):
    """
    Copy the secret data to GCP Secret Manager.
    """

    if multi_key:
        secret_data_str = json.dumps(secret_data)
        create_secret(project_id, secret_id)
        add_secret_version(project_id, secret_id, secret_data_str)
    else:
        for key, value in secret_data.items():
            key = sanitize_key(key)
            create_secret(project_id, f"{secret_id}-{key}")
            add_secret_version(project_id, f"{secret_id}-{key}", value)

def load_config(path):
    """
    Load the configuration file.
    """
    with open(path, 'r') as f:
        return yaml.safe_load(f)

def copy_secrets(project_id, sm_config):
    """
    Copy the secrets to GCP Secret Manager.
    """
    if "collection" in sm_config:
        collection = sm_config["collection"]
        for target in collection:
            if "source" in collection[target]:
                source = collection[target]["source"]
                if "/" in source:
                    source_type, source_name = source.split("/")
                else:
                    source_type = "secret"
                    source_name = source
                if source_type not in ["secret", "configmap"]:
                    print(f"Unsupported source type: {source_type} for target: {target}")
                    continue
            if "namespace" in collection[target]:
                namespace = collection[target]["namespace"]
            else:
                print(f"Namespace not provided for target: {target}")
                continue

            multi_key = True if "multiKey" not in collection[target] or collection[target]["multiKey"] else False

            if source_type == "secret":
                secret_data = load_k8s_secret(namespace, source_name)
                if secret_data:
                    copy_secret_to_sm(project_id, target, secret_data, multi_key)
                else:
                    print(f"Secret {source_name} not found in namespace {namespace}")
            elif source_type == "configmap":
                configmap_data = load_k8s_configmap(namespace, source_name)
                if configmap_data:
                    copy_secret_to_sm(project_id, target, configmap_data, multi_key)
                else:
                    print(f"ConfigMap {source_name} not found in namespace {namespace}")

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--project-id', default=os.getenv('PROJECT_ID', ''))
    parser.add_argument('--config-file', default=os.getenv('CONFIG_FILE', '/etc/secrets/config.yaml'))
    args = parser.parse_args()

    if not args.project_id:
        parser.error('Please provide the project ID using --project-id')

    sm_config = load_config(args.config_file)
    copy_secrets(args.project_id, sm_config)
